/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */
package org.apache.poi.xwpf.usermodel;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.poi.POIXMLDocumentPart;
import org.apache.poi.POIXMLException;
import org.apache.poi.openxml4j.exceptions.OpenXML4JException;
import org.apache.poi.openxml4j.opc.PackagePart;
import org.apache.poi.openxml4j.opc.PackageRelationship;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlOptions;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTComment;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTComments;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTP;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTbl;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTc;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CommentsDocument;

public class XWPFCommentsDocument extends POIXMLDocumentPart implements IBody {
    protected List<XWPFComment> comments = new ArrayList<XWPFComment>();
    protected CommentsDocument commentsDocument;
    List<XWPFParagraph> paragraphs = new ArrayList<XWPFParagraph>(1);
    List<XWPFTable> tables= new ArrayList<XWPFTable>(1);
    List<XWPFPictureData> pictures = new ArrayList<XWPFPictureData>();
    List<IBodyElement> bodyElements = new ArrayList<IBodyElement>(1);
	
	/**
	 *create a new XWPFCommentsDocument object with an existing document 
	 */
	public XWPFCommentsDocument(PackagePart part, PackageRelationship rel) throws IOException, OpenXML4JException{
		super(part, rel);
	}

	/**
	 * create a new XWPFCommentsDocument object for use in a new document
	 */
	public XWPFCommentsDocument(){
		comments = new ArrayList<XWPFComment>();
		commentsDocument = CommentsDocument.Factory.newInstance();		
	}

	/**
	 * read comments from an existing package
	 */
	@Override
	protected void onDocumentRead() throws IOException{
		InputStream is;
		is = getPackagePart().getInputStream();
		try {
			commentsDocument = CommentsDocument.Factory.parse(is);
			for (CTComment ctComment : commentsDocument.getComments().getCommentList()) {
                comments.add(new XWPFComment(ctComment, this));
            }
		} catch (XmlException e) {
			throw new POIXMLException();
		}
	}
	
	/**
	 * returns the list of comments in the comments document
	 *
	 * @return
	 */
	
    public XWPFComment[] getComments() {
        return comments.toArray(new XWPFComment[comments.size()]);
    }
    
    /**
     * adds a new comment
     */
    public XWPFComment addNewComment()
    {
        CTComments ctc = commentsDocument.getComments(); 
        if (ctc == null) {
            ctc = commentsDocument.addNewComments();
        }
        CTComment ctComment = ctc.addNewComment();
        XWPFComment xComment = new XWPFComment(ctComment, this);
        
        return xComment;
    }
    
    /**
     * returns a comments based on its id
     *
     * @return
     */
        
    public XWPFComment getCommentByID(int id) {
        Iterator<XWPFComment> iter = comments.iterator();
        while (iter.hasNext()) {
            XWPFComment comment = iter.next();
            if (comment.getId() == id)
                return comment;
        }

        return null;
    }
	
	/**
	 * save and commit comments
	 */
	@Override
    protected void commit() throws IOException {
        XmlOptions xmlOptions = new XmlOptions(DEFAULT_XML_OPTIONS);
        //xmlOptions.setSaveSyntheticDocumentElement(new QName(CTDocument1.type.getName().getNamespaceURI(), "comments"));
        Map<String,String> map = new HashMap<String,String>();
        map.put("http://schemas.openxmlformats.org/markup-compatibility/2006", "ve");
        map.put("urn:schemas-microsoft-com:office:office", "o");
        map.put("http://schemas.openxmlformats.org/officeDocument/2006/relationships", "r");
        map.put("http://schemas.openxmlformats.org/officeDocument/2006/math", "m");
        map.put("urn:schemas-microsoft-com:vml", "v");
        map.put("http://schemas.openxmlformats.org/drawingml/2006/wordprocessingDrawing", "wp");
        map.put("urn:schemas-microsoft-com:office:word", "w10");
        map.put("http://schemas.openxmlformats.org/wordprocessingml/2006/main", "w");
        map.put("http://schemas.microsoft.com/office/word/2006/wordml", "wne");
        xmlOptions.setSaveSuggestedPrefixes(map);
        PackagePart part = getPackagePart();
        OutputStream out = part.getOutputStream();
        commentsDocument.save(out, xmlOptions);
        out.close();
    }

    /**
     * returns the Part, to which the body belongs, which you need for adding relationship to other parts
     * @see org.apache.poi.xwpf.usermodel.IBody#getPart()
     */
	public POIXMLDocumentPart getPart() {
        return this;
    }

    public List<IBodyElement> getBodyElements() {
        // TODO Auto-generated method stub
        return null;
    }

    /**
     * Returns the paragraph(s) that holds
     *  the text of the comment.
     */
    public List<XWPFParagraph> getParagraphs() {
        return Collections.unmodifiableList(paragraphs);
    }

    /**
     * Return the table(s) that holds the text
     *  of the comment
     */
    public List<XWPFTable> getTables()throws ArrayIndexOutOfBoundsException {
        return Collections.unmodifiableList(tables);
    }

    /**
     * if there is a corresponding {@link XWPFParagraph} of the parameter ctTable in the paragraphList of this comment
     * the method will return this paragraph
     * if there is no corresponding {@link XWPFParagraph} the method will return null 
     * @param p is instance of CTP and is searching for an XWPFParagraph
     * @return null if there is no XWPFParagraph with an corresponding CTPparagraph in the paragraphList of this header or footer
     *         XWPFParagraph with the corresponding CTP p
     */
    public XWPFParagraph getParagraph(CTP p){
        for (XWPFParagraph paragraph : paragraphs) {
            if(paragraph.getCTP().equals(p))
                return paragraph;
        }
        return null;

    }

    /**
     * if there is a corresponding {@link XWPFTable} of the parameter ctTable in the tableList of this comment
     * the method will return this table
     * if there is no corresponding {@link XWPFTable} the method will return null 
     * @param ctTable
     */
    public XWPFTable getTable(CTTbl ctTable){
        for (XWPFTable table : tables) {
            if(table==null)
                return null;
            if(table.getCTTbl().equals(ctTable))
                return table;
        }
        return null;
    }

    /**
     * get a List of all Paragraphs
     * @return a list of {@link XWPFParagraph} 
     */
    public List<XWPFParagraph> getListParagraph(){
        return paragraphs;
    }
    
    /**
     * Returns the table at position pos
     * @see org.apache.poi.xwpf.usermodel.IBody#getTableArray(int)
     */
    public XWPFTable getTableArray(int pos) {

        if(pos > 0 && pos < tables.size()){
            return tables.get(pos);
        }
        return null;
    }


    public XWPFParagraph insertNewParagraph(XmlCursor cursor) {
        // TODO Auto-generated method stub
        return null;
    }

    public XWPFTable insertNewTbl(XmlCursor cursor) {
        // TODO Auto-generated method stub
        return null;
    }

    public void insertTable(int pos, XWPFTable table) {
        // TODO Auto-generated method stub
        
    }

    public XWPFTableCell getTableCell(CTTc cell) {
        // TODO Auto-generated method stub
        return null;
    }

    public XWPFDocument getXWPFDocument() {
        return (XWPFDocument)getParent();
    }

    public BodyType getPartType() {
        return BodyType.COMMENTSDOCUMENT;
    }

    public XWPFParagraph getParagraphArray(int pos) {
        // TODO Auto-generated method stub
        return null;
    }
}
	
